@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Module to manage AiiDAlab apps."""
 
 import errno
@@ -41,6 +40,7 @@ from .utils import (
     load_app_registry_entry,
     load_app_registry_index,
     run_pip_install,
+    run_post_install_script,
     run_reentry_scan,
     run_verdi_daemon_restart,
     split_git_url,
@@ -223,7 +223,7 @@ class _AiidaLabApp:
 
     def find_matching_releases(self, specifier, prereleases=None):
         matching_releases = list(
-            specifier.filter(self.releases, prereleases=prereleases)
+            specifier.filter(self.releases or [], prereleases=prereleases)
         )
         # Sort by intrinsic order (e.g. 1.1.0 -> 1.0.1 -> 1.0.0 and so on)
         matching_releases.sort(key=parse, reverse=True)
@@ -262,15 +262,16 @@ class _AiidaLabApp:
     def _install_dependencies(self, python_bin, stdout=None):
         """Try to install the app dependencies with pip (if specified)."""
 
-        def _pip_install(*args, stdout=None):
+        def _pip_install(*args, stdout):
             # The implementation of this function is taken and adapted from:
             # https://www.endpoint.com/blog/2015/01/getting-realtime-output-using-python/
-            stdout = stdout or sys.stdout
 
             # Install package dependencies.
             process = run_pip_install(*args, python_bin=python_bin)
             for line in io.TextIOWrapper(process.stdout, encoding="utf-8"):
                 stdout.write(line)
+            if process.returncode != 0:
+                raise RuntimeError("Failed to install dependencies.")
 
             # AiiDA plugins require reentry run to be found by AiiDA.
             process = run_reentry_scan()
@@ -281,14 +282,14 @@ class _AiidaLabApp:
             process = run_verdi_daemon_restart()
             for line in io.TextIOWrapper(process.stdout, encoding="utf-8"):
                 stdout.write(line)
+            if process.returncode != 0:
+                raise RuntimeError("Failed to restart verdi daemon.")
 
         for path in (self.path.joinpath(".aiidalab"), self.path):
             if path.exists():
                 try:
                     if path.joinpath("setup.py").is_file():
-                        _pip_install(
-                            "--use-feature=in-tree-build", str(path), stdout=stdout
-                        )
+                        _pip_install(str(path), stdout=stdout)
                     elif path.joinpath("requirements.txt").is_file():
                         _pip_install(
                             f"--requirement={path.joinpath('requirements.txt')}",
@@ -302,6 +303,22 @@ class _AiidaLabApp:
                 except CalledProcessError:
                     raise RuntimeError("Failed to install dependencies.")
 
+    def _post_install_triggers(self):
+        """Run a post_install script.
+
+        Typically used to execute additional commands after the app installation.
+        """
+        post_install_file = self.path.joinpath("post_install")
+        if post_install_file.exists():
+            logger.info(f"Running post_install script: {post_install_file}")
+            # We do not track of the output for the execution of the
+            # post_install script, because it may prevent intentional
+            # detachment of child processes. For example, an app might trigger
+            # a background process that is supposed to finish after the
+            # installation is technically completed.
+            if run_post_install_script(post_install_file).returncode != 0:
+                raise CalledProcessError
+
     def _install_from_path(self, path):
         if path.is_dir():
             shutil.copytree(this_or_only_subdir(path), self.path)
@@ -334,8 +351,9 @@ class _AiidaLabApp:
         version=None,
         python_bin=None,
         install_dependencies=True,
-        stdout=None,
+        stdout=sys.stdout,
         prereleases=False,
+        post_install_triggers=True,
     ):
         if version is None:
             try:
@@ -378,6 +396,11 @@ class _AiidaLabApp:
             # Install dependencies
             if install_dependencies:
                 self._install_dependencies(python_bin, stdout=stdout)
+
+            # Run post-installation triggers.
+            if post_install_triggers:
+                self._post_install_triggers()
+
         except RuntimeError as error:
             try:
                 if trash_path is None:
@@ -737,7 +760,7 @@ class AiidaLabApp(traitlets.HasTraits):
     def _get_from_metadata(self, what):
         """Get information from metadata."""
         try:
-            return "{}".format(self._app.metadata[what])
+            return f"{self._app.metadata[what]}"
         except KeyError:
             return f'Field "{what}" is not present in app metadata.'
 
@@ -760,7 +783,7 @@ class AiidaLabApp(traitlets.HasTraits):
 
     @property
     def more(self):
-        return """<a href=./single_app.ipynb?app={}>Manage App</a>""".format(self.name)
+        return f"""<a href=./single_app.ipynb?app={self.name}>Manage App</a>"""
 
     @property
     def _repo(self):
